@@ -9,7 +9,6 @@ mod xdg;
 #[cfg(not(windows))]
 use crate::xdg as dirs;
 
-use std::collections::HashMap;
 use std::path::{Path, PathBuf};
 use std::process::ExitCode;
 use std::sync::{Arc, Mutex};
@@ -26,7 +25,7 @@ use log::{debug, error, info, warn};
 use reqwest::header::{HeaderMap, HeaderName, HeaderValue};
 use reqwest::{Client, RequestBuilder, StatusCode, Url};
 use rss::{Channel, ChannelBuilder, GuidBuilder, ItemBuilder};
-use serde::Deserialize;
+use serde::{Deserialize, Serialize};
 use simple_eyre::eyre;
 
 use crate::dirs::Dirs;
@@ -55,10 +54,27 @@ struct FeedConfig {
     url: String,
     item: String,
     heading: String,
+    link: Option<String>,
     summary: Option<String>,
     date: Option<String>,
 }
 
+#[derive(Debug, Serialize)]
+struct RequestCacheWrite<'a> {
+    headers: Vec<(&'a str, &'a str)>,
+    version: &'a str,
+}
+
+#[derive(Debug, Deserialize)]
+struct RequestCacheRead {
+    headers: Vec<(String, String)>,
+    /// The version of rsspls that created this request cache
+    ///
+    /// May be missing if the cache was created by an older version.
+    #[serde(default)]
+    version: Option<String>,
+}
+
 const RSSPLS_LOG: &str = "RSSPLS_LOG";
 
 #[tokio::main]
@@ -181,7 +197,7 @@ async fn process(
     }?;
     let cached_headers = deserialise_cached_headers(&cache_path);
 
-    process_feed(&client, &feed, &cached_headers)
+    process_feed(client, feed, &cached_headers)
         .await
         .and_then(|ref process_result| {
             match process_result {
@@ -194,6 +210,7 @@ async fn process(
 
                     // Update the cache
                     if let Some(headers) = headers {
+                        debug!("write cache {}", cache_path.display());
                         fs::write(cache_path, headers).wrap_err("unable to write to cache")?;
                     }
 
@@ -263,13 +280,25 @@ async fn process_feed(
         ));
     }
 
+    if config.link.is_none() {
+        info!(
+            "no explicit link selector provided, falling back to heading selector: {:?}",
+            config.heading
+        );
+    }
+
+    let link_selector = config.link.as_ref().unwrap_or(&config.heading);
+
     // Collect the headers for later
     let headers: Vec<_> = resp
         .headers()
         .iter()
         .filter_map(|(name, value)| value.to_str().ok().map(|val| (name.as_str(), val)))
         .collect();
-    let map: HashMap<_, _> = [("headers", headers)].into_iter().collect();
+    let map = RequestCacheWrite {
+        headers,
+        version: version(),
+    };
     let serialised_headers = toml::to_string(&map)
         .map_err(|err| warn!("unable to serialise headers: {}", err))
         .ok();
@@ -289,20 +318,27 @@ async fn process_feed(
         let title = item
             .as_node()
             .select_first(&config.heading)
-            .map_err(|()| eyre!("invalid selector for title: {}", config.heading))?;
+            .map_err(|()| eyre!("invalid selector for heading: {}", config.heading))?;
+        let link = item
+            .as_node()
+            .select_first(link_selector)
+            .map_err(|()| eyre!("invalid selector for link: {}", link_selector))?;
         // TODO: Need to make links absolute (probably ones in content too)
-        let attrs = title.attributes.borrow();
-        let link = attrs
+        let attrs = link.attributes.borrow();
+        let link_url = attrs
             .get("href")
-            .ok_or_else(|| eyre!("element selected as heading has no 'href' attribute"))?;
+            .ok_or_else(|| eyre!("element selected as link has no 'href' attribute"))?;
         let title_text = title.text_contents();
         let description = extract_description(config, &item, &title_text)?;
         let date = extract_pub_date(config, &item)?;
-        let guid = GuidBuilder::default().value(link).permalink(false).build();
+        let guid = GuidBuilder::default()
+            .value(link_url)
+            .permalink(false)
+            .build();
 
         let rss_item = ItemBuilder::default()
             .title(title_text)
-            .link(base_url.parse(link).ok().map(|u| u.to_string()))
+            .link(base_url.parse(link_url).ok().map(|u| u.to_string()))
             .guid(Some(guid))
             .pub_date(date.map(|date| date.to_rfc2822()))
             .description(description)
@@ -385,7 +421,7 @@ fn parse_date(node: &NodeDataRef<ElementData>) -> Option<DateTime<FixedOffset>>
         .flatten()
         .and_then(|datetime| {
             debug!("trying datetime attribute");
-            anydate::parse(trim_date(&datetime)).ok()
+            anydate::parse(trim_date(datetime)).ok()
         })
         .map(|x| {
             debug!("using datetime attribute");
@@ -439,10 +475,22 @@ fn extract_description(
 
 fn deserialise_cached_headers(path: &Path) -> Option<HeaderMap<HeaderValue>> {
     let raw = fs::read(path).ok()?;
-    let mut map: HashMap<String, Vec<(String, String)>> = toml::from_slice(&raw).ok()?;
-    let pairs = map.remove("headers")?;
+    let cache: RequestCacheRead = toml::from_slice(&raw).ok()?;
+
+    if cache.version.as_deref() != Some(version()) {
+        debug!(
+            "cache version ({:?}) != to this version ({:?}), ignoring cache at: {}",
+            cache.version,
+            version(),
+            path.display()
+        );
+        return None;
+    }
+
+    debug!("using cache at: {}", path.display());
     Some(
-        pairs
+        cache
+            .headers
             .into_iter()
             .filter_map(|(name, value)| {
                 HeaderName::try_from(name)
@@ -454,11 +502,11 @@ fn deserialise_cached_headers(path: &Path) -> Option<HeaderMap<HeaderValue>> {
 }
 
 pub fn version_string() -> String {
-    format!(
-        "{} version {}",
-        env!("CARGO_PKG_NAME"),
-        env!("CARGO_PKG_VERSION")
-    )
+    format!("{} version {}", env!("CARGO_PKG_NAME"), version())
+}
+
+fn version() -> &'static str {
+    env!("CARGO_PKG_VERSION")
 }
 
 #[cfg(test)]
